window.settings = window.domElements = window.objects = window.utils = window.variables = window.canvasElements = window.globals = window.events = window.constants = {};
objects.intro = objects.game = objects.records = objects.server = undefined;
variables.playerName = variables.currentScore = undefined;
variables.totalImages = variables.imagesLoaded = undefined;
variables.hintBtnImage = variables.submitBtnImage = undefined;
variables.blockSize = variables.leftRightMargin = undefined;
variables.leftLimit = variables.rightLimit = variables.topLimit = variables.bottomLimit = undefined;
constants.GAME_WIDTH = 600;
constants.GAME_HEIGHT = 500;
constants.GAME_MIN_WIDTH = 150;
constants.GAME_MIN_HEIGHT = 150;
constants.MILISECONDS_FOR_FRAME = 1000 / 25;
constants.SCREENS_ANIMATION_DURATION = 500;

domElements.$introScreen = $("#intro_screen");
domElements.$gameScreen = $("#game_screen");
domElements.$recordScreen = $("#records_screen");
domElements.$preloader = $("#preloader");
domElements.$introGameName = $("#intro_game_name");
domElements.$canvas = $("#canvas");
domElements.invisibleClassName = "invisible";

utils.turnOnScreen = function($screen) {
    var $oldScreen, newZIndex;
    newZIndex = 3;
    if (typeof domElements.$activeScreen !== "undefined") {
        $oldScreen = domElements.$activeScreen;
        newZIndex = parseInt($oldScreen.css("z-index")) + 1;
    }
    domElements.$activeScreen = $screen;
    $screen.css("z-index", newZIndex);
    $screen.css("left", $(window).width() * -1);
    $screen.animate({left: 0}, constants.SCREENS_ANIMATION_DURATION, "swing", function() {
        if (($oldScreen != null) && $oldScreen != domElements.$activeScreen) $oldScreen.css({"left": "-2000px"});
    });
    utils.redrawDomBlockObject($screen);
}

utils.redrawDomBlockObject = function($object) {
    $object.css("display", "none");
    $object[0].offsetHeight;
    $object.css("display", "block");
}

utils.resizeAll = function() {
    utils.resizeCanvas();
}

utils.resizeCanvas = function() {
    var $canvasContainer = domElements.$gameScreen;
    var containerWidth = Math.max($canvasContainer.width(), constants.GAME_MIN_WIDTH);
    var containerHeight = Math.max($canvasContainer.height(), constants.GAME_MIN_HEIGHT);
    var ratio = constants.GAME_WIDTH / constants.GAME_HEIGHT;
    var marginLeft = 0;
    var marginTop = 0;
    var zoom = 1;

    var tempWidth = containerWidth;
    zoom = tempWidth / constants.GAME_WIDTH;
    var tempHeight = tempWidth / ratio;
    if(tempHeight > containerHeight) {
        tempHeight = containerHeight;
        tempWidth = containerHeight * ratio;
        zoom = tempHeight / constants.GAME_HEIGHT;
        marginLeft = (containerWidth - tempWidth) / 2;
    }else {
        marginTop = (containerHeight - tempHeight) / 2;
    }

    domElements.$canvas.width(tempWidth);
    domElements.$canvas.height(tempHeight);
    paper.view.viewSize = new paper.Size(tempWidth, tempHeight);
    paper.view.zoom = zoom;
    paper.view.size.width = tempWidth;
    paper.view.size.height = tempHeight;
    paper.view.center = new paper.Point(tempWidth / 2, tempHeight / 2);

    domElements.$canvas.css("margin-top", marginTop-20 + "px");
    domElements.$canvas.css("margin-left", marginLeft + "px");
    paper.view.draw();
}

utils.getRandomWords = function() {
    var arr = settings.WORDS.slice();
    var words = new Array();
    while (words.length < settings.ONE_GAME_WORDS_COUNT) {
        var index = Math.round(Math.random() * (arr.length - 1));
        words.push(arr[index]);
        arr.splice(index, 1);
    }
    return words;
}

utils.calculateBlockSizeAndMargin = function() {
    var spaceForBlocks = constants.GAME_WIDTH - settings.ANSWER_SIDE_MARGIN * 2 - settings.ANSWER_BLOCKS_MARGIN *
            (objects.game.levelWord.length - 1);
    var blockSize = spaceForBlocks / objects.game.levelWord.length;
    variables.blockSize = Math.min(blockSize, settings.ANSWER_MAX_SIZE);
    variables.leftRightMargin = settings.ANSWER_SIDE_MARGIN;
    if(blockSize > variables.blockSize) {
        variables.leftRightMargin = (constants.GAME_WIDTH - (variables.blockSize * objects.game.levelWord.length +
                settings.ANSWER_BLOCKS_MARGIN * (objects.game.levelWord.length - 1))) / 2;
    }

    variables.leftLimit = variables.topLimit = variables.blockSize / 2;
    variables.rightLimit = constants.GAME_WIDTH - variables.blockSize / 2;
    variables.bottomLimit = constants.GAME_HEIGHT - variables.blockSize / 2 - 60;
}

// RESIZE
utils.setOrientationChangeListener = function() {
    $(window).bind('orientationchange', function(e){
        e.preventDefault();
        utils.pauseAndRedrawGame();
    });
}

utils.pauseAndRedrawGame = function() {
    objects.game.pauseGame(constants.MILISECONDS_FOR_FRAME);
    utils.redrawDomBlockObject(domElements.$activeScreen);
}