function AnswerBlock(letter, index) {
    this.letter = letter;
    this.index = index;

    this.rect = this.hint = undefined;
    this.currentVariant = this.stars = undefined;

    this.setView();
}

AnswerBlock.prototype.setView = function() {
    this.rect = new paper.Path.Rectangle(new paper.Point(0, 0), new paper.Size(variables.blockSize, variables.blockSize));
    this.rect.strokeWidth = 5;
    this.rect.strokeColor = settings.BLOCK_BORDER_COLOR;
    this.rect.fillColor = settings.ANSWER_FILL_COLOR;
    this.rect.position = new paper.Point(variables.leftRightMargin + variables.blockSize * this.index + settings.ANSWER_BLOCKS_MARGIN
            * this.index + variables.blockSize / 2, settings.ANSWER_MARGIN_TOP + variables.blockSize / 2);
}

AnswerBlock.prototype.clear = function() {
    this.rect.remove();
    if(typeof(this.hint) != "undefined") this.hint.remove();
    if(typeof(this.stars) == "undefined") return;
    for(var i = 0; i < this.stars.length; i++) {
        this.stars[i].clear();
    }
    this.stars = undefined;
}

AnswerBlock.prototype.setHint = function() {
    this.hint = new paper.PointText({
        content: this.letter,
        fillColor: 'black',
        fontFamily: 'Times new roman',
        fontWeight: 'bold',
        fontSize: 24
    });
    this.hint.position = new paper.Point(this.rect.position.x, this.rect.position.y - variables.blockSize / 2
            - settings.ANSWER_MARGIN_TOP / 2);
}

AnswerBlock.prototype.spawnStars = function() {
    this.stars = new Array();
    var starsCount = settings.STARS_FOR_ANSWER + Math.round(Math.random() * (settings.STARS_VARIATIVITY * 2)) - settings.STARS_VARIATIVITY;
    for(var i = 0; i < starsCount; i++) {
        var startX = this.rect.position.x + Math.round(Math.random() * variables.blockSize) - variables.blockSize / 2;
        var startY = this.rect.position.y + Math.round(Math.random() * variables.blockSize) - variables.blockSize / 2;
        var star = new Star(startX, startY);
        this.stars.push(star);
    }
}

AnswerBlock.prototype.setSelection = function() {
    this.rect.strokeWidth = variables.blockSize / 8;
    this.rect.strokeColor = settings.ANSWER_FILL_COLOR;
    this.rect.opacity = 1;
}

AnswerBlock.prototype.setSelectionOff = function() {
    this.rect.strokeWidth = 5;
    this.rect.strokeColor = settings.BLOCK_BORDER_COLOR;
    this.rect.opacity = 1;
}

AnswerBlock.prototype.onFrame = function() {
    if(typeof (this.stars) == "undefined") return;
    for(var i = 0; i < this.stars.length; i++) this.stars[i].onFrame();
}

AnswerBlock.prototype.isOver = function(clickPoint) {
    if(clickPoint.x > this.rect.position.x - variables.blockSize / 2 && clickPoint.x < this.rect.position.x
            + variables.blockSize / 2 && clickPoint.y > this.rect.position.y - variables.blockSize / 2 &&
            clickPoint.y < this.rect.position.y + variables.blockSize / 2) {
        return true;
    }
    return false;
}