function Game() {
    this.$recordsPopup = $("#records_popup");
    this.$popupPointsParagraph = $("#points_paragraph");
    this.$popupPlayAgainBtn = $("#records_popup_btn");
    this.$popupPlayAgainBtn.click(function(e) {
        objects.game.$recordsPopup.addClass(domElements.invisibleClassName);
        objects.game.startNewGame();
        utils.turnOnScreen(domElements.$gameScreen);
    });

    this.$submitBlock = $("#submit_question_block");
    this.$yesSubmitBtn = $("#yes_submit_btn");
    this.$noSubmitBtn = $("#no_submit_btn");

    this.$totalPointsSpan = $("#total_points_span");
    this.$levelPointsSpan = $("#level_points_span");

    this.words = this.levelWord = undefined;
    this.variantBlocks = this.answerBlocks = undefined;

    this.tick = this.timer = this.clearTimeout = this.pauseTimeout = undefined; //RESIZE
    this.background = undefined;
    this.buttons = undefined;
    this.points = this.levelPoints = undefined;
    this.draggableVariant = undefined;
    this.wordIsReady = false;
    this.starTimer = undefined;
    this.newPoint = undefined;

    this.clickDifferenceX = this.clickDifferenceY = undefined;
    this.overAnswer = undefined;

    this.setView();
    this.setListeners();
}

Game.prototype.setView = function() {
    this.background = new paper.Path.Rectangle(new paper.Point(0, 0), new paper.Size(constants.GAME_WIDTH, constants.GAME_HEIGHT));
    this.background.fillColor = settings.BACKGROUND_COLOR;
    this.buttons = new Buttons();
}
Game.prototype.setListeners = function() {
    this.$noSubmitBtn.click(function(e) {
        objects.game.$submitBlock.addClass(domElements.invisibleClassName);
    });
    this.$yesSubmitBtn.click(function(e) {
        objects.game.$submitBlock.addClass(domElements.invisibleClassName);
        objects.game.endGame();
    });
}

Game.prototype.setClick = function(clickPoint) {
//    $("#point_span").text("CLICKED");
    this.newPoint = undefined;
    if(this.buttons.submitIsClicked(clickPoint)) {
        this.$submitBlock.removeClass(domElements.invisibleClassName);
        return;
    }

    if(this.wordIsReady) return;

    if(this.buttons.hintIsClicked(clickPoint)) {
        if(this.buttons.hintBtn.visible == false) return;
        this.callHint();
        return;
    }

    if(typeof (this.draggableVariant) != "undefined") return;

    for(var i = 0; i < this.variantBlocks.length; i++) {
        if(this.variantBlocks[i].isClicked(clickPoint)) {
            this.draggableVariant = this.variantBlocks[i];
            this.clickDifferenceX = clickPoint.x - this.variantBlocks[i].img.position.x;
            this.clickDifferenceY = clickPoint.y - this.variantBlocks[i].img.position.y;
            break
        }
    }

    if(typeof (this.draggableVariant) == "undefined" || typeof (this.draggableVariant.currentAnswerBlock) == "undefined") return;
    this.draggableVariant.currentAnswerBlock.currentVariant = undefined;
    this.draggableVariant.currentAnswerBlock.setSelectionOff();
    this.draggableVariant.currentAnswerBlock = undefined;
}

Game.prototype.setMovement = function(currentPoint) {
    if(typeof (this.draggableVariant) == "undefined" || this.wordIsReady) return;
    this.newPoint = currentPoint;
}

Game.prototype.setFrameDrag = function() {
    if(typeof (this.draggableVariant) == "undefined" || this.wordIsReady || typeof (this.newPoint) == "undefined") return;
    this.draggableVariant.move(this.newPoint);
    this.clearOverAnswer();
    for(var i = 0; i < this.answerBlocks.length; i++) {
        if(typeof (this.answerBlocks[i].currentVariant) != "undefined") continue;
        if(this.answerBlocks[i].isOver(this.draggableVariant.trianglePoint)) {
            this.overAnswer = this.answerBlocks[i];
            this.overAnswer.rect.fillColor = settings.ANSWER_OVER_COLOR;
            this.overAnswer.rect.strokeColor = settings.ANSWER_BORDER_COLOR;
            break;
        }
    }
    this.newPoint = undefined;
}

Game.prototype.clearOverAnswer = function() {
    if(typeof (this.overAnswer) != "undefined") {
        this.overAnswer.rect.fillColor = settings.ANSWER_FILL_COLOR;
        this.overAnswer.rect.strokeColor = settings.BLOCK_BORDER_COLOR;

        this.overAnswer = undefined;
    }
}

Game.prototype.stopDrag = function() {
    this.newPoint = undefined;
    if(this.wordIsReady || typeof (this.draggableVariant) == "undefined") return;
    this.setVariantToAnswer();
    this.clearOverAnswer();
    this.draggableVariant = undefined;
    this.checkIfWordIsReady();
}

Game.prototype.checkIfWordIsReady = function() {
    for(var i = 0; i < this.answerBlocks.length; i++) {
        if(typeof (this.answerBlocks[i].currentVariant) == "undefined" || this.answerBlocks[i].currentVariant.letter !=
                this.answerBlocks[i].letter) return;
    }

    this.wordIsReady = true;
    this.points += this.levelPoints;
    this.setTexts();

    this.starTimer = 0;
    for(var i = 0; i < this.answerBlocks.length; i++) {
        this.answerBlocks[i].spawnStars();
    }
}

Game.prototype.setVariantToAnswer = function() {
    if(typeof (this.overAnswer) == "undefined") return;
    this.overAnswer.currentVariant = this.draggableVariant;
    this.overAnswer.setSelection();
    this.draggableVariant.currentAnswerBlock = this.overAnswer;
    this.draggableVariant.img.position = this.overAnswer.rect.position;
}

Game.prototype.callHint = function() {
    this.buttons.setHintClick();
    this.levelPoints -= 10;
    if(this.levelPoints < 0) this.levelPoints = 0;
    this.setTexts();
    this.answerBlocks[Math.round(Math.random() * (this.answerBlocks.length - 1))].setHint();
}

Game.prototype.startNewGame = function() {
    this.words = utils.getRandomWords();
    this.points = 0;
    this.tick = 0;
    this.timer = window.setInterval(function() {
        objects.game.onFrame();
    }, constants.MILISECONDS_FOR_FRAME);
    this.startNewLevel();
}

Game.prototype.startNewLevel = function() {
    if(this.words.length == 0) {
        this.endGame();
        return;
    }
    this.wordIsReady = false;
    this.starTimer = undefined;
    this.levelWord = this.words[0].toUpperCase();
    this.words.splice(0, 1);
    utils.calculateBlockSizeAndMargin();
    this.setAnswers();
    this.setVariants();
    this.buttons.hintBtn.visible = true;
    this.levelPoints = settings.START_LEVEL_POINTS;
    this.setTexts();
}

Game.prototype.setAnswers = function() {
    if(typeof (this.answerBlocks) != "undefined") this.clearAnswers();
    this.answerBlocks = new Array();
    for(var i = 0; i < this.levelWord.length; i++) {
        var answer = new AnswerBlock(this.levelWord.charAt(i), i);
        this.answerBlocks.push(answer);
    }
}

Game.prototype.setVariants = function() {
    if(typeof (this.variantBlocks) != "undefined") this.clearVariants();
    this.variantBlocks = new Array();
    var tempArr = new Array();
    for(var i = 0; i < this.answerBlocks.length; i++) tempArr.push(this.answerBlocks[i]);

    var notFirstIndex = Math.round(Math.random() * (tempArr.length - 2)) + 1;
    var firstVariant = new VariantBlock(tempArr[notFirstIndex].letter, 0);
    this.variantBlocks.push(firstVariant);
    tempArr.splice(notFirstIndex, 1);

    var variantIndex = 1;
    while(tempArr.length > 0) {
        var index = Math.round(Math.random() * (tempArr.length - 1));
        var variant = new VariantBlock(tempArr[index].letter, variantIndex);
        this.variantBlocks.push(variant);
        tempArr.splice(index, 1);
        variantIndex++;
    }
}

Game.prototype.clearAnswers = function() {
    for(var i = 0; i < this.answerBlocks.length; i++) {
        this.answerBlocks[i].clear();
    }
    this.answerBlocks = undefined;
}

Game.prototype.clearVariants = function() {
    for(var i = 0; i < this.variantBlocks.length; i++) {
        this.variantBlocks[i].clear();
    }
    this.variantBlocks = undefined;
}

// RESIZE
Game.prototype.pauseGame = function(pauseMilliseconds) {
    if(typeof (this.timer) == "undefined") return;
    window.clearInterval(this.timer);
    this.timer = undefined;
    this.pauseTimeout = window.setTimeout(function(){objects.game.resumeGame();}, pauseMilliseconds);
}

Game.prototype.resumeGame = function() {
    window.clearTimeout(this.pauseTimeout);
    this.pauseTimeout = undefined;
    this.timer = window.setInterval(function() {objects.game.onFrame();}, constants.MILISECONDS_FOR_FRAME);
}
//

Game.prototype.onFrame = function() {
    this.tick += constants.MILISECONDS_FOR_FRAME;
    if(this.tick % 1000 == 0 && this.levelPoints > 0 && !this.wordIsReady) {
        this.levelPoints--;
        this.setTexts();
    }
    this.buttons.onFrame();

    this.setFrameDrag();

    if(typeof (this.starTimer) != "undefined") {
        this.starTimer += constants.MILISECONDS_FOR_FRAME;
        for(var i = 0; i < this.answerBlocks.length; i++) this.answerBlocks[i].onFrame();
        if(this.starTimer >= settings.STAR_TIME * 1000) {
            this.starTimer = undefined;
            this.startNewLevel();
        }
    }

    paper.view.draw();
}

Game.prototype.endGame = function() {
    window.clearInterval(this.timer);
    this.timer = undefined;
    this.clearTimer = window.setTimeout(function() {
        objects.game.removeGameObjects();
    }, constants.MILISECONDS_FOR_FRAME * 2);
}

Game.prototype.removeGameObjects = function() {
    window.clearTimeout(this.clearTimer);
    this.clearTimer = undefined;
    variables.currentScore = this.points;

    this.clearAnswers();
    this.clearVariants();
    this.buttons.clear();

    if(settings.RECORDS_ENABLED) {
        objects.records.setNewRecords();
        utils.turnOnScreen(domElements.$recordScreen);
    }else {
        this.$recordsPopup.removeClass(domElements.invisibleClassName);
        this.$popupPointsParagraph.text(this.points);
        //RECORDS
    }
}

Game.prototype.setTexts = function() {
    this.$totalPointsSpan.text(this.points);
    this.$levelPointsSpan.text(this.levelPoints);
}